@@ -34,8 +34,8 @@
 # Version
 # =======
 import sys, time
-__version__= "2021.04.12"
-__moniker__ = "Nelson hopped brut cider"
+__version__= "2022.8.31"
+__moniker__ = "Materia"
 #datetime.datetime.utcnow().strftime("%Y.%m")
 __author__ = "Josh L. Espinoza"
 __email__ = "jespinoz@jcvi.org, jol.espinoz@gmail.com"
@@ -89,7 +89,7 @@ _submodules = ["core", "classification", "db", "feature_extraction", "hierarchy"
 _core = ["Dataset"]
 _hierarchy = ["Agglomerative", "Topology"]
 _networks = ["Hive", "TemporalNetwork","EnsembleAssociationNetwork", "SampleSpecificPerturbationNetwork", "DifferentialEnsembleAssociationNetwork"]
-_ordination = ["PrincipalComponentAnalysis", "PrincipalCoordinatesAnalysis", "Manifold"]
+_ordination = ["PrincipalComponentAnalysis", "PrincipalCoordinatesAnalysis", "Manifold", "Procrustes"]
 _symmetry = ["Symmetric"]
 _utils = ["Chromatic", "Suppress", "pv"]
 
@@ -111,4 +111,17 @@ print(" * Hive NetworkX v{}".format(hx.__version__), file=sys.stderr)
 __ecosystem__["hive_networkx"] = hx.__version__
 import ensemble_networkx as enx
 print(" * Ensemble NetworkX v{}".format(enx.__version__), file=sys.stderr)
-__ecosystem__["ensemble_networkx"] = enx.__version__
\ No newline at end of file
+__ecosystem__["ensemble_networkx"] = enx.__version__
+
+print(" ", file=sys.stderr)
+print(utils.format_header("Auxillary Ecosystem"), file=sys.stderr)
+print(" * Python v{}".format(sys.version.split(" ")[0]), file=sys.stderr)
+
+import pandas as pd
+print(" * Pandas v{}".format(pd.__version__), file=sys.stderr)
+import numpy as np
+print(" * NumPy v{}".format(np.__version__), file=sys.stderr)
+# import scipy as sp
+# print(" * SciPy v{}".format(sp.__version__), file=sys.stderr)
+# import matplotlib as mpl
+# print(" * Matplotlib v{}".format(mpl.__version__), file=sys.stderr)
\ No newline at end of file
