@@ -3,7 +3,8 @@
 # ==============================================================================
 # Built-ins
 import os, sys, time, datetime, logging, uuid, pathlib, posix, gzip, bz2, zipfile, subprocess, requests, operator
-from collections import OrderedDict, defaultdict, Mapping
+from collections import OrderedDict, defaultdict
+from collections.abc import Mapping 
 from io import TextIOWrapper
 
 # PyData
@@ -25,11 +26,10 @@ from sklearn.datasets import load_iris
 import soothsayer_utils as syu
 
 functions_from_soothsayer_utils = [
-'assert_acceptable_arguments', 'boolean', 'consecutive_replace', 'contains', 'dict_build', 'dict_collapse', 'dict_expand', 'dict_fill', 'dict_filter', 'dict_reverse', 'dict_tree',"dict_py_to_bash",
+'assert_acceptable_arguments', 'boolean', 'consecutive_replace', 'contains', 'dict_build', 'dict_collapse', 'dict_expand', 'dict_fill', 'dict_reverse', 'dict_tree', 'dict_filter',
 'flatten', 'format_memory', 'format_duration', 'format_header', 'format_path', 'fragment',  'get_timestamp', 'get_unique_identifier', 'hash_kmer', 'infer_compression', 'is_all_same_type',
 'is_dict', 'is_dict_like', 'is_file_like', 'is_function', 'is_in_namespace', 'is_nonstring_iterable', 'is_symmetrical', 'is_number', 'is_path_like', 'is_query_class', "is_graph", "is_color",
-'iterable_depth', 'join_as_strings',
-'pad_left', 'pv', 'range_like',  'reverse_complement', 'to_precision', "check_packages", "Suppress","get_color_midpoint","remove_pairwise_nan", "format_pairwise",
+'iterable_depth', 'join_as_strings', 'pad_left', 'pv', 'range_like',  'reverse_complement', 'to_precision', "check_packages", "Suppress","get_color_midpoint","remove_pairwise_nan", "format_pairwise","intersection", "union","iterate_nested_dict",
 ]
 
 __all__ = {
@@ -824,8 +824,9 @@ def get_parameters_ellipse(df_xy, groups, metric=np.mean, n_std=3, into=pd.DataF
             order = vals.argsort()[::-1]
             return vals[order], vecs[:,order]
         # Data
-        x =  df_xy.iloc[:,0][idx].as_matrix()
-        y =  df_xy.iloc[:,1][idx].as_matrix()
+        idx = sorted(idx)
+        x =  df_xy.iloc[:,0][idx].values
+        y =  df_xy.iloc[:,1][idx].values
         # Reduce
         x_reduce = metric(x)
         y_reduce = metric(y)
@@ -883,7 +884,7 @@ def determine_mode_for_logfiles(path, force_overwrite):
 # Pandas
 # ===========
 # Prepend level to pandas index
-def pd_prepend_level_to_index(x, level_value, level_name=None):
+def pd_prepend_level_to_index(x, level_value, level_name=None, axis=0):
     """
     Prepend a level to a pd.Series or pd.DataFrame
     
@@ -910,7 +911,11 @@ def pd_prepend_level_to_index(x, level_value, level_name=None):
     Adapted from the following source:
     https://stackoverflow.com/questions/14744068/prepend-a-level-to-a-pandas-multiindex
     """
-    return pd.concat([x], keys=[level_value], names=[level_name])
+    assert_acceptable_arguments(query=axis, target={0,1})
+    if axis == 0:
+        return pd.concat([x], keys=[level_value], names=[level_name])
+    if axis == 1:
+        return pd.concat([x.T], keys=[level_value], names=[level_name]).T
 
 # Collapse a pandas series
 def pd_series_collapse(u:pd.Series, into=pd.Series, type_collection=set):
